@@ -326,7 +326,7 @@ proc_done(struct gotd_child_proc *proc)
 
 	if (proc->iev.ibuf.fd != -1) {
 		event_del(&proc->iev.ev);
-		msgbuf_clear(&proc->iev.ibuf.w);
+		imsgbuf_clear(&proc->iev.ibuf);
 		close(proc->iev.ibuf.fd);
 	}
 
@@ -384,7 +384,7 @@ disconnect(struct gotd_client *client)
 
 	slot = client_hash(client->id) % nitems(gotd_clients);
 	STAILQ_REMOVE(&gotd_clients[slot], client, gotd_client, entry);
-	imsg_clear(&client->iev.ibuf);
+	imsgbuf_clear(&client->iev.ibuf);
 	event_del(&client->iev.ev);
 	evtimer_del(&client->tmo);
 	if (client->fd != -1)
@@ -404,9 +404,12 @@ disconnect_on_error(struct gotd_client *client, const struct got_error *err)
 	if (err->code != GOT_ERR_EOF) {
 		log_warnx("uid %d: %s", client->euid, err->msg);
 		if (client->fd != -1) {
-			imsg_init(&ibuf, client->fd);
-			gotd_imsg_send_error(&ibuf, 0, PROC_GOTD, err);
-			imsg_clear(&ibuf);
+			if (imsgbuf_init(&ibuf, client->fd) != -1) {
+				gotd_imsg_send_error(&ibuf, 0, PROC_GOTD,
+				    err);
+				imsgbuf_clear(&ibuf);
+			} else
+				log_warn("%s: imsgbuf_init failed", __func__);
 		}
 	}
 	disconnect(client);
@@ -520,7 +523,6 @@ send_info(struct gotd_client *client)
 static const struct got_error *
 stop_gotd(struct gotd_client *client)
 {
-
 	if (client->euid != 0)
 		return got_error_set_errno(EPERM, "stop");
 
@@ -593,48 +595,51 @@ gotd_request(int fd, short events, void *arg)
 	ssize_t n;
 
 	if (events & EV_WRITE) {
-		while (ibuf->w.queued) {
-			n = msgbuf_write(&ibuf->w);
-			if (n == -1 && errno == EPIPE) {
-				/*
-				 * The client has closed its socket.
-				 * This can happen when Git clients are
-				 * done sending pack file data.
-				 */
-				msgbuf_clear(&ibuf->w);
-				continue;
-			} else if (n == -1 && errno != EAGAIN) {
-				err = got_error_from_errno("msgbuf_write");
-				disconnect_on_error(client, err);
-				return;
-			}
-			if (n == 0) {
-				/* Connection closed. */
-				err = got_error(GOT_ERR_EOF);
-				disconnect_on_error(client, err);
+		err = gotd_imsg_flush(ibuf);
+		if (err) {
+			/*
+			 * The client has closed its socket.  This can
+			 * happen when Git clients are done sending
+			 * pack file data.
+			 */
+			if (err->code == GOT_ERR_ERRNO && errno == EPIPE) {
+				disconnect(client);
 				return;
 			}
+			disconnect_on_error(client, err);
+			return;
 		}
 
-		/* Disconnect gotctl(8) now that messages have been sent. */
-		if (!client_is_reading(client) && !client_is_writing(client)) {
+		/* Disconnect gotctl(8) if all messages have been sent. */
+		if (!client_is_reading(client) && !client_is_writing(client) &&
+		    imsgbuf_queuelen(ibuf) == 0) {
 			disconnect(client);
 			return;
 		}
 	}
 
-	if ((events & EV_READ) == 0)
-		return;
-
-	memset(&imsg, 0, sizeof(imsg));
+	if (events & EV_READ) {
+		n = imsgbuf_read(ibuf);
+		if (n == -1) {
+			err = got_error_from_errno("imsgbuf_read");
+			disconnect_on_error(client, err);
+			return;
+		}
+		if (n == 0) {
+			 err = got_error(GOT_ERR_EOF);
+			 disconnect_on_error(client, err);
+			 return;
+		}
+	}
 
 	while (err == NULL) {
-		err = gotd_imsg_recv(&imsg, ibuf, 0);
-		if (err) {
-			if (err->code == GOT_ERR_PRIVSEP_READ)
-				err = NULL;
+		n = imsg_get(ibuf, &imsg);
+		if (n == -1) {
+			err = got_error_from_errno("imsg_get");
 			break;
 		}
+		if (n == 0)
+			break;
 
 		evtimer_del(&client->tmo);
 
@@ -680,7 +685,6 @@ recv_connect(uint32_t *client_id, struct imsg *imsg)
 	const struct got_error *err = NULL;
 	struct gotd_imsg_connect iconnect;
 	size_t datalen;
-	int s = -1;
 	struct gotd_client *client = NULL;
 
 	*client_id = 0;
@@ -690,12 +694,6 @@ recv_connect(uint32_t *client_id, struct imsg *imsg)
 		return got_error(GOT_ERR_PRIVSEP_LEN);
 	memcpy(&iconnect, imsg->data, sizeof(iconnect));
 
-	s = imsg_get_fd(imsg);
-	if (s == -1) {
-		err = got_error(GOT_ERR_PRIVSEP_NO_FD);
-		goto done;
-	}
-
 	if (find_client(iconnect.client_id)) {
 		err = got_error_msg(GOT_ERR_CLIENT_ID, "duplicate client ID");
 		goto done;
@@ -711,13 +709,20 @@ recv_connect(uint32_t *client_id, struct imsg *imsg)
 
 	client->state = GOTD_CLIENT_STATE_NEW;
 	client->id = iconnect.client_id;
-	client->fd = s;
-	s = -1;
 	/* The auth process will verify UID/GID for us. */
 	client->euid = iconnect.euid;
 	client->egid = iconnect.egid;
 
-	imsg_init(&client->iev.ibuf, client->fd);
+	client->fd = imsg_get_fd(imsg);
+	if (client->fd == -1) {
+		err = got_error(GOT_ERR_PRIVSEP_NO_FD);
+		goto done;
+	}
+	if (imsgbuf_init(&client->iev.ibuf, client->fd) == -1) {
+		err = got_error_from_errno("imsgbuf_init");
+		goto done;
+	}
+	imsgbuf_allow_fdpass(&client->iev.ibuf);
 	client->iev.handler = gotd_request;
 	client->iev.events = EV_READ;
 	client->iev.handler_arg = client;
@@ -732,7 +737,7 @@ recv_connect(uint32_t *client_id, struct imsg *imsg)
 	log_debug("%s: new client uid %d connected on fd %d", __func__,
 	    client->euid, client->fd);
 done:
-	if (err) {
+	if (err && client) {
 		struct gotd_child_proc *listen_proc = gotd.listen_proc;
 		struct gotd_imsg_disconnect idisconnect;
 
@@ -742,8 +747,9 @@ done:
 		    &idisconnect, sizeof(idisconnect)) == -1)
 			log_warn("imsg compose DISCONNECT");
 
-		if (s != -1)
-			close(s);
+		if (client->fd != -1)
+			close(client->fd);
+		free(client);
 	}
 
 	return err;
@@ -770,7 +776,7 @@ kill_proc(struct gotd_child_proc *proc, int fatal)
 
 	if (proc->iev.ibuf.fd != -1) {
 		event_del(&proc->iev.ev);
-		msgbuf_clear(&proc->iev.ibuf.w);
+		imsgbuf_clear(&proc->iev.ibuf);
 		close(proc->iev.ibuf.fd);
 		proc->iev.ibuf.fd = -1;
 	}
@@ -1057,6 +1063,7 @@ connect_repo_child(struct gotd_client *client,
 static void
 gotd_dispatch_listener(int fd, short event, void *arg)
 {
+	const struct got_error *err = NULL;
 	struct gotd_imsgev *iev = arg;
 	struct imsgbuf *ibuf = &iev->ibuf;
 	struct gotd_child_proc *proc = gotd.listen_proc;
@@ -1068,7 +1075,7 @@ gotd_dispatch_listener(int fd, short event, void *arg)
 		fatalx("%s: unexpected fd %d", __func__, fd);
 
 	if (event & EV_READ) {
-		if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
+		if ((n = imsgbuf_read(ibuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0) {
 			/* Connection closed. */
@@ -1078,14 +1085,9 @@ gotd_dispatch_listener(int fd, short event, void *arg)
 	}
 
 	if (event & EV_WRITE) {
-		n = msgbuf_write(&ibuf->w);
-		if (n == -1 && errno != EAGAIN)
-			fatal("msgbuf_write");
-		if (n == 0) {
-			/* Connection closed. */
-			shut = 1;
-			goto done;
-		}
+		err = gotd_imsg_flush(ibuf);
+		if (err)
+			fatalx("%s", err->msg);
 	}
 
 	for (;;) {
@@ -1144,6 +1146,7 @@ done:
 static void
 gotd_dispatch_notifier(int fd, short event, void *arg)
 {
+	const struct got_error *err = NULL;
 	struct gotd_imsgev *iev = arg;
 	struct imsgbuf *ibuf = &iev->ibuf;
 	struct gotd_child_proc *proc = gotd.notify_proc;
@@ -1155,7 +1158,7 @@ gotd_dispatch_notifier(int fd, short event, void *arg)
 		fatalx("%s: unexpected fd %d", __func__, fd);
 
 	if (event & EV_READ) {
-		if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
+		if ((n = imsgbuf_read(ibuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0) {
 			/* Connection closed. */
@@ -1165,14 +1168,9 @@ gotd_dispatch_notifier(int fd, short event, void *arg)
 	}
 
 	if (event & EV_WRITE) {
-		n = msgbuf_write(&ibuf->w);
-		if (n == -1 && errno != EAGAIN)
-			fatal("msgbuf_write");
-		if (n == 0) {
-			/* Connection closed. */
-			shut = 1;
-			goto done;
-		}
+		err = gotd_imsg_flush(ibuf);
+		if (err)
+			fatalx("%s", err->msg);
 	}
 
 	for (;;) {
@@ -1234,7 +1232,7 @@ gotd_dispatch_auth_child(int fd, short event, void *arg)
 		fatalx("cannot find auth child process for fd %d", fd);
 
 	if (event & EV_READ) {
-		if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
+		if ((n = imsgbuf_read(ibuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0) {
 			/* Connection closed. */
@@ -1244,13 +1242,9 @@ gotd_dispatch_auth_child(int fd, short event, void *arg)
 	}
 
 	if (event & EV_WRITE) {
-		n = msgbuf_write(&ibuf->w);
-		if (n == -1 && errno != EAGAIN)
-			fatal("msgbuf_write");
-		if (n == 0) {
-			/* Connection closed. */
-			shut = 1;
-		}
+		err = gotd_imsg_flush(ibuf);
+		if (err)
+			fatalx("%s", err->msg);
 		goto done;
 	}
 
@@ -1380,8 +1374,8 @@ connect_session(struct gotd_client *client)
 	 * We are no longer interested in messages from this client.
 	 * Further client requests will be handled by the session process.
 	 */
-	msgbuf_clear(&client->iev.ibuf.w);
-	imsg_clear(&client->iev.ibuf);
+	imsgbuf_clear(&client->iev.ibuf);
+	imsgbuf_clear(&client->iev.ibuf);
 	event_del(&client->iev.ev);
 	client->fd = -1; /* will be closed via copy in client->iev.ibuf.fd */
 
@@ -1391,6 +1385,7 @@ connect_session(struct gotd_client *client)
 static void
 gotd_dispatch_client_session(int fd, short event, void *arg)
 {
+	const struct got_error *err = NULL;
 	struct gotd_imsgev *iev = arg;
 	struct imsgbuf *ibuf = &iev->ibuf;
 	struct gotd_child_proc *proc = NULL;
@@ -1408,7 +1403,7 @@ gotd_dispatch_client_session(int fd, short event, void *arg)
 	}
 
 	if (event & EV_READ) {
-		if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
+		if ((n = imsgbuf_read(ibuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0) {
 			/* Connection closed. */
@@ -1418,14 +1413,9 @@ gotd_dispatch_client_session(int fd, short event, void *arg)
 	}
 
 	if (event & EV_WRITE) {
-		n = msgbuf_write(&ibuf->w);
-		if (n == -1 && errno != EAGAIN)
-			fatal("msgbuf_write");
-		if (n == 0) {
-			/* Connection closed. */
-			shut = 1;
-			goto done;
-		}
+		err = gotd_imsg_flush(ibuf);
+		if (err)
+			fatalx("%s", err->msg);
 	}
 
 	proc = client->session;
@@ -1556,6 +1546,7 @@ connect_notifier_and_session(struct gotd_client *client)
 static void
 gotd_dispatch_repo_child(int fd, short event, void *arg)
 {
+	const struct got_error *err = NULL;
 	struct gotd_imsgev *iev = arg;
 	struct imsgbuf *ibuf = &iev->ibuf;
 	struct gotd_child_proc *proc = NULL;
@@ -1573,7 +1564,7 @@ gotd_dispatch_repo_child(int fd, short event, void *arg)
 	}
 
 	if (event & EV_READ) {
-		if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
+		if ((n = imsgbuf_read(ibuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0) {
 			/* Connection closed. */
@@ -1583,14 +1574,9 @@ gotd_dispatch_repo_child(int fd, short event, void *arg)
 	}
 
 	if (event & EV_WRITE) {
-		n = msgbuf_write(&ibuf->w);
-		if (n == -1 && errno != EAGAIN)
-			fatal("msgbuf_write");
-		if (n == 0) {
-			/* Connection closed. */
-			shut = 1;
-			goto done;
-		}
+		err = gotd_imsg_flush(ibuf);
+		if (err)
+			fatalx("%s", err->msg);
 	}
 
 	proc = client->repo;
@@ -1598,7 +1584,6 @@ gotd_dispatch_repo_child(int fd, short event, void *arg)
 		fatalx("cannot find child process for fd %d", fd);
 
 	for (;;) {
-		const struct got_error *err = NULL;
 		uint32_t client_id = 0;
 		int do_disconnect = 0;
 
@@ -1616,9 +1601,11 @@ gotd_dispatch_repo_child(int fd, short event, void *arg)
 			err = connect_session(client);
 			if (err)
 				break;
-			err = connect_notifier_and_session(client);
-			if (err)
-				break;
+			if (client_is_writing(client)) {
+				err = connect_notifier_and_session(client);
+				if (err)
+					break;
+			}
 			err = connect_repo_child(client, proc);
 			break;
 		default:
@@ -1751,7 +1738,9 @@ start_listener(char *argv0, const char *confpath, int daemonize, int verbosity)
 
 	proc->pid = start_child(proc->type, NULL, argv0, confpath,
 	    proc->pipe[1], daemonize, verbosity);
-	imsg_init(&proc->iev.ibuf, proc->pipe[0]);
+	if (imsgbuf_init(&proc->iev.ibuf, proc->pipe[0]) == -1)
+		fatal("imsgbuf_init");
+	imsgbuf_allow_fdpass(&proc->iev.ibuf);
 	proc->iev.handler = gotd_dispatch_listener;
 	proc->iev.events = EV_READ;
 	proc->iev.handler_arg = NULL;
@@ -1785,7 +1774,9 @@ start_notifier(char *argv0, const char *confpath, int daemonize, int verbosity)
 
 	proc->pid = start_child(proc->type, NULL, argv0, confpath,
 	    proc->pipe[1], daemonize, verbosity);
-	imsg_init(&proc->iev.ibuf, proc->pipe[0]);
+	if (imsgbuf_init(&proc->iev.ibuf, proc->pipe[0]) == -1)
+		fatal("imsgbuf_init");
+	imsgbuf_allow_fdpass(&proc->iev.ibuf);
 	proc->iev.handler = gotd_dispatch_notifier;
 	proc->iev.events = EV_READ;
 	proc->iev.handler_arg = NULL;
@@ -1829,7 +1820,9 @@ start_session_child(struct gotd_client *client, struct gotd_repo *repo,
 		fatal("socketpair");
 	proc->pid = start_child(proc->type, proc->repo_path, argv0,
 	    confpath, proc->pipe[1], daemonize, verbosity);
-	imsg_init(&proc->iev.ibuf, proc->pipe[0]);
+	if (imsgbuf_init(&proc->iev.ibuf, proc->pipe[0]) == -1)
+		fatal("imsgbuf_init");
+	imsgbuf_allow_fdpass(&proc->iev.ibuf);
 	log_debug("proc %s %s is on fd %d",
 	    gotd_proc_names[proc->type], proc->repo_path,
 	    proc->pipe[0]);
@@ -1883,7 +1876,9 @@ start_repo_child(struct gotd_client *client, enum gotd_procid proc_type,
 		fatal("socketpair");
 	proc->pid = start_child(proc->type, proc->repo_path, argv0,
 	    confpath, proc->pipe[1], daemonize, verbosity);
-	imsg_init(&proc->iev.ibuf, proc->pipe[0]);
+	if (imsgbuf_init(&proc->iev.ibuf, proc->pipe[0]) == -1)
+		fatal("imsgbuf_init");
+	imsgbuf_allow_fdpass(&proc->iev.ibuf);
 	log_debug("proc %s %s is on fd %d",
 	    gotd_proc_names[proc->type], proc->repo_path,
 	    proc->pipe[0]);
@@ -1945,7 +1940,9 @@ start_auth_child(struct gotd_client *client, int required_auth,
 		fatal("socketpair");
 	proc->pid = start_child(proc->type, proc->repo_path, argv0,
 	    confpath, proc->pipe[1], daemonize, verbosity);
-	imsg_init(&proc->iev.ibuf, proc->pipe[0]);
+	if (imsgbuf_init(&proc->iev.ibuf, proc->pipe[0]) == -1)
+		fatal("imsgbuf_init");
+	imsgbuf_allow_fdpass(&proc->iev.ibuf);
 	log_debug("proc %s %s is on fd %d",
 	    gotd_proc_names[proc->type], proc->repo_path,
 	    proc->pipe[0]);
@@ -2490,8 +2487,9 @@ main(int argc, char **argv)
 		if (imsg_compose(imsgbuf, GOTD_IMSG_SECRETS, 0, 0, -1,
 		    &n, sizeof(n)) == -1)
 			fatal("imsg_compose GOTD_IMSG_SECRETS");
-		if (imsg_flush(imsgbuf))
-			fatal("imsg_flush");
+		error = gotd_imsg_flush(imsgbuf);
+		if (error)
+			fatalx("%s", error->msg);
 
 		for (i = 0; i < n; ++i) {
 			struct iovec iov[5];
@@ -2516,8 +2514,9 @@ main(int argc, char **argv)
 			if (imsg_composev(imsgbuf, GOTD_IMSG_SECRET,
 			    0, 0, -1, iov, 5) == -1)
 				fatal("imsg_composev GOTD_IMSG_SECRET");
-			if (imsg_flush(imsgbuf))
-				fatal("imsg_flush");
+			error = gotd_imsg_flush(imsgbuf);
+			if (error)
+				fatalx("%s", error->msg);
 		}
 
 		gotd_secrets_free(gotd.secrets);
