@@ -372,6 +372,7 @@ done:
 static void
 notify_dispatch_session(int fd, short event, void *arg)
 {
+	const struct got_error *err = NULL;
 	struct gotd_imsgev *iev = arg;
 	struct imsgbuf *ibuf = &iev->ibuf;
 	ssize_t n;
@@ -379,7 +380,7 @@ notify_dispatch_session(int fd, short event, void *arg)
 	struct imsg imsg;
 
 	if (event & EV_READ) {
-		if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
+		if ((n = imsgbuf_read(ibuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0) {
 			/* Connection closed. */
@@ -389,11 +390,10 @@ notify_dispatch_session(int fd, short event, void *arg)
 	}
 
 	if (event & EV_WRITE) {
-		n = msgbuf_write(&ibuf->w);
-		if (n == -1 && errno != EAGAIN && errno != EPIPE)
-			fatal("msgbuf_write");
-		if (n == 0 || (n == -1 && errno == EPIPE)) {
-			/* Connection closed. */
+		err = gotd_imsg_flush(ibuf);
+		if (err) {
+			if (err->code != GOT_ERR_ERRNO || errno != EPIPE)
+				fatalx("%s", err->msg);
 			shut = 1;
 			goto done;
 		}
@@ -428,7 +428,7 @@ done:
 
 		/* This pipe is dead. Remove its event handler */
 		event_del(&iev->ev);
-		imsg_clear(&iev->ibuf);
+		imsgbuf_clear(&iev->ibuf);
 
 		session = find_session_by_fd(fd);
 		if (session)
@@ -458,7 +458,11 @@ recv_session(struct imsg *imsg)
 	}
 
 	session->id = get_session_id();
-	imsg_init(&session->iev.ibuf, fd);
+	if (imsgbuf_init(&session->iev.ibuf, fd) == -1) {
+		close(fd);
+		return got_error_from_errno("imsgbuf_init");
+	}
+	imsgbuf_allow_fdpass(&session->iev.ibuf);
 	session->iev.handler = notify_dispatch_session;
 	session->iev.events = EV_READ;
 	session->iev.handler_arg = NULL;
@@ -500,6 +504,7 @@ notify_ibuf_get_str(char **ret, struct ibuf *ibuf)
 static void
 notify_dispatch(int fd, short event, void *arg)
 {
+	const struct got_error *err = NULL;
 	struct gotd_imsgev *iev = arg;
 	struct imsgbuf *imsgbuf = &iev->ibuf;
 	ssize_t n;
@@ -509,7 +514,7 @@ notify_dispatch(int fd, short event, void *arg)
 	struct gotd_secret *s;
 
 	if (event & EV_READ) {
-		if ((n = imsg_read(imsgbuf)) == -1 && errno != EAGAIN)
+		if ((n = imsgbuf_read(imsgbuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0) {
 			/* Connection closed. */
@@ -519,14 +524,9 @@ notify_dispatch(int fd, short event, void *arg)
 	}
 
 	if (event & EV_WRITE) {
-		n = msgbuf_write(&imsgbuf->w);
-		if (n == -1 && errno != EAGAIN)
-			fatal("msgbuf_write");
-		if (n == 0) {
-			/* Connection closed. */
-			shut = 1;
-			goto done;
-		}
+		err = gotd_imsg_flush(imsgbuf);
+		if (err)
+			fatalx("%s", err->msg);
 	}
 
 	for (;;) {
@@ -626,7 +626,10 @@ notify_main(const char *title, struct gotd_repolist *repos,
 	signal_add(&evsighup, NULL);
 	signal_add(&evsigusr1, NULL);
 
-	imsg_init(&gotd_notify.parent_iev.ibuf, GOTD_FILENO_MSG_PIPE);
+	if (imsgbuf_init(&gotd_notify.parent_iev.ibuf, GOTD_FILENO_MSG_PIPE)
+	    == -1)
+		fatal("imsgbuf_init");
+	imsgbuf_allow_fdpass(&gotd_notify.parent_iev.ibuf);
 	gotd_notify.parent_iev.handler = notify_dispatch;
 	gotd_notify.parent_iev.events = EV_READ;
 	gotd_notify.parent_iev.handler_arg = NULL;
