@@ -116,7 +116,7 @@ disconnect(struct gotd_session_client *client)
 	    GOTD_IMSG_DISCONNECT, PROC_SESSION_WRITE, -1, NULL, 0) == -1)
 		log_warn("imsg compose DISCONNECT");
 
-	imsg_clear(&gotd_session.repo_child_iev.ibuf);
+	imsgbuf_clear(&gotd_session.repo_child_iev.ibuf);
 	event_del(&gotd_session.repo_child_iev.ev);
 	evtimer_del(&client->tmo);
 	close(client->fd);
@@ -145,9 +145,13 @@ disconnect_on_error(struct gotd_session_client *client,
 
 	if (err->code != GOT_ERR_EOF) {
 		log_warnx("uid %d: %s", client->euid, err->msg);
-		imsg_init(&ibuf, client->fd);
-		gotd_imsg_send_error(&ibuf, 0, PROC_SESSION_WRITE, err);
-		imsg_clear(&ibuf);
+		if (imsgbuf_init(&ibuf, client->fd) == -1) {
+			log_warn("imsgbuf_init");
+		} else {
+			gotd_imsg_send_error(&ibuf, 0, PROC_SESSION_WRITE,
+			    err);
+			imsgbuf_clear(&ibuf);
+		}
 	}
 
 	disconnect(client);
@@ -859,6 +863,7 @@ recv_notification_content(struct imsg *imsg)
 static void
 session_dispatch_repo_child(int fd, short event, void *arg)
 {
+	const struct got_error *err = NULL;
 	struct gotd_imsgev *iev = arg;
 	struct imsgbuf *ibuf = &iev->ibuf;
 	struct gotd_session_client *client = &gotd_session_client;
@@ -867,7 +872,7 @@ session_dispatch_repo_child(int fd, short event, void *arg)
 	struct imsg imsg;
 
 	if (event & EV_READ) {
-		if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
+		if ((n = imsgbuf_read(ibuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0) {
 			/* Connection closed. */
@@ -877,14 +882,9 @@ session_dispatch_repo_child(int fd, short event, void *arg)
 	}
 
 	if (event & EV_WRITE) {
-		n = msgbuf_write(&ibuf->w);
-		if (n == -1 && errno != EAGAIN)
-			fatal("msgbuf_write");
-		if (n == 0) {
-			/* Connection closed. */
-			shut = 1;
-			goto done;
-		}
+		err = gotd_imsg_flush(ibuf);
+		if (err)
+			fatalx("%s", err->msg);
 	}
 
 	for (;;) {
@@ -1209,27 +1209,22 @@ session_dispatch_client(int fd, short events, void *arg)
 	ssize_t n;
 
 	if (events & EV_WRITE) {
-		while (ibuf->w.queued) {
-			n = msgbuf_write(&ibuf->w);
-			if (n == -1 && errno == EPIPE) {
-				/*
-				 * The client has closed its socket.
-				 * This can happen when Git clients are
-				 * done sending pack file data.
-				 */
-				msgbuf_clear(&ibuf->w);
-				continue;
-			} else if (n == -1 && errno != EAGAIN) {
-				err = got_error_from_errno("imsg_flush");
-				disconnect_on_error(client, err);
-				return;
-			}
-			if (n == 0) {
-				/* Connection closed. */
-				err = got_error(GOT_ERR_EOF);
-				disconnect_on_error(client, err);
+		err = gotd_imsg_flush(ibuf);
+		if (err) {
+			/*
+			 * The client has closed its socket.  This can
+			 * happen when Git clients are done sending
+			 * pack file data.
+			 * Pending notifications should still be sent.
+			 */
+			if (STAILQ_FIRST(&notifications) != NULL)
+				return; 
+			if (err->code == GOT_ERR_ERRNO && errno == EPIPE) {
+				disconnect(client);
 				return;
 			}
+			disconnect_on_error(client, err);
+			return;
 		}
 
 		if (client->flush_disconnect) {
@@ -1238,30 +1233,36 @@ session_dispatch_client(int fd, short events, void *arg)
 		}
 	}
 
-	if ((events & EV_READ) == 0)
-		return;
-
-	memset(&imsg, 0, sizeof(imsg));
+	if (events & EV_READ) {
+		n = imsgbuf_read(ibuf);
+		if (n == -1) {
+			err = got_error_from_errno("imsgbuf_read");
+			disconnect_on_error(client, err);
+			return;
+		}
+		if (n == 0) {
+			/*
+			 * The client has closed its socket.  This can
+			 * happen when Git clients are done sending
+			 * pack file data.
+			 * Pending notifications should still be sent.
+			 */
+			if (STAILQ_FIRST(&notifications) != NULL)
+				return; 
+			err = got_error(GOT_ERR_EOF);
+			disconnect_on_error(client, err);
+			return;
+		}
+	}
 
 	while (err == NULL) {
-		err = gotd_imsg_recv(&imsg, ibuf, 0);
-		if (err) {
-			if (err->code == GOT_ERR_PRIVSEP_READ)
-				err = NULL;
-			else if (err->code == GOT_ERR_EOF &&
-			    gotd_session.state ==
-			    GOTD_STATE_EXPECT_CAPABILITIES) {
-				/*
-				 * The client has closed its socket before
-				 * sending its capability announcement.
-				 * This can happen when Git clients have
-				 * no ref-updates to send.
-				 */
-				disconnect_on_error(client, err);
-				return;
-			}
+		n = imsg_get(ibuf, &imsg);
+		if (n == -1) {
+			err = got_error_from_errno("imsg_get");
 			break;
 		}
+		if (n == 0)
+			break;
 
 		evtimer_del(&client->tmo);
 
@@ -1418,7 +1419,9 @@ recv_connect(struct imsg *imsg)
 	if (client->username == NULL)
 		return got_error_from_errno("strndup");
 
-	imsg_init(&client->iev.ibuf, client->fd);
+	if (imsgbuf_init(&client->iev.ibuf, client->fd) == -1)
+		return got_error_from_errno("imsgbuf_init");
+	imsgbuf_allow_fdpass(&client->iev.ibuf);
 	client->iev.handler = session_dispatch_client;
 	client->iev.events = EV_READ;
 	client->iev.handler_arg = NULL;
@@ -1444,7 +1447,7 @@ session_dispatch_notifier(int fd, short event, void *arg)
 	struct gotd_session_notif *notif;
 
 	if (event & EV_READ) {
-		if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
+		if ((n = imsgbuf_read(ibuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0) {
 			/* Connection closed. */
@@ -1454,14 +1457,9 @@ session_dispatch_notifier(int fd, short event, void *arg)
 	}
 
 	if (event & EV_WRITE) {
-		n = msgbuf_write(&ibuf->w);
-		if (n == -1 && errno != EAGAIN)
-			fatal("msgbuf_write");
-		if (n == 0) {
-			/* Connection closed. */
-			shut = 1;
-			goto done;
-		}
+		err = gotd_imsg_flush(ibuf);
+		if (err)
+			fatalx("%s", err->msg);
 	}
 
 	for (;;) {
@@ -1502,8 +1500,7 @@ done:
 	} else {
 		/* This pipe is dead. Remove its event handler */
 		event_del(&iev->ev);
-		imsg_clear(&iev->ibuf);
-		imsg_init(&iev->ibuf, -1);
+		imsgbuf_clear(&iev->ibuf);
 	}
 }
 
@@ -1530,7 +1527,11 @@ recv_notifier(struct imsg *imsg)
 	if (fd == -1)
 		return NULL; /* notifications unused */
 
-	imsg_init(&iev->ibuf, fd);
+	if (imsgbuf_init(&iev->ibuf, fd) == -1) {
+		close(fd);
+		return got_error_from_errno("imsgbuf_init");
+	}
+	imsgbuf_allow_fdpass(&iev->ibuf);
 	iev->handler = session_dispatch_notifier;
 	iev->events = EV_READ;
 	iev->handler_arg = NULL;
@@ -1570,7 +1571,11 @@ recv_repo_child(struct imsg *imsg)
 	if (fd == -1)
 		return got_error(GOT_ERR_PRIVSEP_NO_FD);
 
-	imsg_init(&gotd_session.repo_child_iev.ibuf, fd);
+	if (imsgbuf_init(&gotd_session.repo_child_iev.ibuf, fd) == -1) {
+		close(fd);
+		return got_error_from_errno("imsgbuf_init");
+	}
+	imsgbuf_allow_fdpass(&gotd_session.repo_child_iev.ibuf);
 	gotd_session.repo_child_iev.handler = session_dispatch_repo_child;
 	gotd_session.repo_child_iev.events = EV_READ;
 	gotd_session.repo_child_iev.handler_arg = NULL;
@@ -1589,6 +1594,7 @@ recv_repo_child(struct imsg *imsg)
 static void
 session_dispatch(int fd, short event, void *arg)
 {
+	const struct got_error *err = NULL;
 	struct gotd_imsgev *iev = arg;
 	struct imsgbuf *ibuf = &iev->ibuf;
 	struct gotd_session_client *client = &gotd_session_client;
@@ -1597,7 +1603,7 @@ session_dispatch(int fd, short event, void *arg)
 	struct imsg imsg;
 
 	if (event & EV_READ) {
-		if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
+		if ((n = imsgbuf_read(ibuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0) {
 			/* Connection closed. */
@@ -1607,14 +1613,9 @@ session_dispatch(int fd, short event, void *arg)
 	}
 
 	if (event & EV_WRITE) {
-		n = msgbuf_write(&ibuf->w);
-		if (n == -1 && errno != EAGAIN)
-			fatal("msgbuf_write");
-		if (n == 0) {
-			/* Connection closed. */
-			shut = 1;
-			goto done;
-		}
+		err = gotd_imsg_flush(ibuf);
+		if (err)
+			fatalx("%s", err->msg);
 	}
 
 	for (;;) {
@@ -1692,7 +1693,11 @@ session_write_main(const char *title, const char *repo_path,
 	    sizeof(gotd_session.request_timeout));
 	gotd_session.repo_cfg = repo_cfg;
 
-	imsg_init(&gotd_session.notifier_iev.ibuf, -1);
+	if (imsgbuf_init(&gotd_session.notifier_iev.ibuf, -1) == -1) {
+		err = got_error_from_errno("imsgbuf_init");
+		goto done;
+	}
+	imsgbuf_allow_fdpass(&gotd_session.notifier_iev.ibuf);
 
 	err = got_repo_open(&gotd_session.repo, repo_path, NULL, pack_fds);
 	if (err)
@@ -1728,7 +1733,12 @@ session_write_main(const char *title, const char *repo_path,
 	gotd_session_client.delta_cache_fd = -1;
 	gotd_session_client.accept_flush_pkt = 1;
 
-	imsg_init(&gotd_session.parent_iev.ibuf, GOTD_FILENO_MSG_PIPE);
+	if (imsgbuf_init(&gotd_session.parent_iev.ibuf, GOTD_FILENO_MSG_PIPE)
+	    == -1) {
+		err = got_error_from_errno("imsgbuf_init");
+		goto done;
+	}
+	imsgbuf_allow_fdpass(&gotd_session.parent_iev.ibuf);
 	gotd_session.parent_iev.handler = session_dispatch;
 	gotd_session.parent_iev.events = EV_READ;
 	gotd_session.parent_iev.handler_arg = NULL;
