@@ -43,6 +43,7 @@
 #include "got_reference.h"
 
 #include "gotwebd.h"
+#include "log.h"
 
 __dead void usage(void);
 
@@ -59,12 +60,12 @@ void
 imsg_event_add(struct imsgev *iev)
 {
 	if (iev->handler == NULL) {
-		imsg_flush(&iev->ibuf);
+		imsgbuf_flush(&iev->ibuf);
 		return;
 	}
 
 	iev->events = EV_READ;
-	if (iev->ibuf.w.queued)
+	if (imsgbuf_queuelen(&iev->ibuf))
 		iev->events |= EV_WRITE;
 
 	event_del(&iev->ev);
@@ -104,10 +105,7 @@ main_compose_sockets(struct gotwebd *env, uint32_t type, int fd,
 
 		/* prevent fd exhaustion */
 		if (d != -1) {
-			do {
-				ret = imsg_flush(&env->iev_server[i].ibuf);
-			} while (ret == -1 && errno == EAGAIN);
-			if (ret == -1)
+			if (imsgbuf_flush(&env->iev_server[i].ibuf) == -1)
 				goto err;
 			imsg_event_add(&env->iev_server[i]);
 		}
@@ -143,16 +141,14 @@ gotwebd_dispatch_sockets(int fd, short event, void *arg)
 	ibuf = &iev->ibuf;
 
 	if (event & EV_READ) {
-		if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
+		if ((n = imsgbuf_read(ibuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0)	/* Connection closed */
 			shut = 1;
 	}
 	if (event & EV_WRITE) {
-		if ((n = msgbuf_write(&ibuf->w)) == -1 && errno != EAGAIN)
+		if (imsgbuf_write(ibuf) == -1)
 			fatal("msgbuf_write");
-		if (n == 0)	/* Connection closed */
-			shut = 1;
 	}
 
 	for (;;) {
@@ -209,7 +205,7 @@ gotwebd_sighdlr(int sig, short event, void *arg)
 static int
 spawn_socket_process(struct gotwebd *env, const char *argv0, int n)
 {
-	const char	*argv[5];
+	const char	*argv[8];
 	int		 argc = 0;
 	int		 p[2];
 	pid_t		 pid;
@@ -224,7 +220,9 @@ spawn_socket_process(struct gotwebd *env, const char *argv0, int n)
 		break;
 	default:	/* parent */
 		close(p[0]);
-		imsg_init(&env->iev_server[n].ibuf, p[1]);
+		if (imsgbuf_init(&env->iev_server[n].ibuf, p[1]) == -1)
+			fatal("imsgbuf_init");
+		imsgbuf_allow_fdpass(&env->iev_server[n].ibuf);
 		env->iev_server[n].handler = gotwebd_dispatch_sockets;
 		env->iev_server[n].data = &env->iev_server[n];
 		event_set(&env->iev_server[n].ev, p[1], EV_READ,
@@ -237,9 +235,15 @@ spawn_socket_process(struct gotwebd *env, const char *argv0, int n)
 
 	argv[argc++] = argv0;
 	argv[argc++] = "-S";
+	if (strcmp(env->gotwebd_conffile, GOTWEBD_CONF) != 0) {
+		argv[argc++] = "-f";
+		argv[argc++] = env->gotwebd_conffile;
+	}
 	if (env->gotwebd_debug)
 		argv[argc++] = "-d";
-	if (env->gotwebd_verbose)
+	if (env->gotwebd_verbose > 0)
+		argv[argc++] = "-v";
+	if (env->gotwebd_verbose > 1)
 		argv[argc++] = "-v";
 	argv[argc] = NULL;
 
@@ -272,6 +276,7 @@ main(int argc, char **argv)
 	int			 no_action = 0;
 	int			 server_proc = 0;
 	const char		*conffile = GOTWEBD_CONF;
+	const char		*username = GOTWEBD_DEFAULT_USER;
 	const char		*argv0;
 
 	if ((argv0 = argv[0]) == NULL)
@@ -305,7 +310,8 @@ main(int argc, char **argv)
 			server_proc = 1;
 			break;
 		case 'v':
-			env->gotwebd_verbose++;
+			if (env->gotwebd_verbose < 3)
+				env->gotwebd_verbose++;
 			break;
 		default:
 			usage();
@@ -331,9 +337,11 @@ main(int argc, char **argv)
 	if (geteuid())
 		fatalx("need root privileges");
 
-	pw = getpwnam(GOTWEBD_USER);
+	if (env->user)
+		username = env->user;
+	pw = getpwnam(username);
 	if (pw == NULL)
-		fatalx("unknown user %s", GOTWEBD_USER);
+		fatalx("unknown user %s", username);
 	env->pw = pw;
 
 	log_init(env->gotwebd_debug, LOG_DAEMON);
@@ -343,8 +351,8 @@ main(int argc, char **argv)
 		setproctitle("sockets");
 		log_procinit("sockets");
 
-		if (chroot(pw->pw_dir) == -1)
-			fatal("chroot %s", pw->pw_dir);
+		if (chroot(env->httpd_chroot) == -1)
+			fatal("chroot %s", env->httpd_chroot);
 		if (chdir("/") == -1)
 			fatal("chdir /");
 		if (setgroups(1, &pw->pw_gid) == -1 ||
@@ -438,10 +446,12 @@ gotwebd_configure(struct gotwebd *env)
 	TAILQ_FOREACH(sock, &env->sockets, entry) {
 		if (config_setsock(env, sock) == -1)
 			fatalx("%s: send socket error", __func__);
-		if (config_setfd(env, sock) == -1)
-			fatalx("%s: send priv_fd error", __func__);
 	}
 
+	/* send the temp files */
+	if (config_setfd(env) == -1)
+		fatalx("%s: send priv_fd error", __func__);
+
 	if (main_compose_sockets(env, IMSG_CFG_DONE, -1, NULL, 0) == -1)
 		fatal("main_compose_sockets IMSG_CFG_DONE");
 
@@ -471,7 +481,7 @@ gotwebd_shutdown(void)
 
 	for (i = 0; i < env->nserver; ++i) {
 		event_del(&env->iev_server[i].ev);
-		imsg_clear(&env->iev_server[i].ibuf);
+		imsgbuf_clear(&env->iev_server[i].ibuf);
 		close(env->iev_server[i].ibuf.fd);
 		env->iev_server[i].ibuf.fd = -1;
 	}
