@@ -24,14 +24,6 @@
 #include <limits.h>
 #include <stdio.h>
 
-#include "tmpl.h"
-
-#ifdef DEBUG
-#define dprintf(x...)   do { log_debug(x); } while(0)
-#else
-#define dprintf(x...)
-#endif /* DEBUG */
-
 #ifndef nitems
 #define nitems(_a)	(sizeof((_a)) / sizeof((_a)[0]))
 #endif
@@ -39,7 +31,9 @@
 /* GOTWEBD DEFAULTS */
 #define GOTWEBD_CONF		 "/etc/gotwebd.conf"
 
-#define GOTWEBD_USER		 "www"
+#ifndef GOTWEBD_DEFAULT_USER
+#define GOTWEBD_DEFAULT_USER	 "www"
+#endif
 
 #define GOTWEBD_MAXDESCRSZ	 1024
 #define GOTWEBD_MAXCLONEURLSZ	 1024
@@ -208,7 +202,6 @@ struct transport {
 	unsigned int		 repos_total;
 	unsigned int		 next_disp;
 	unsigned int		 prev_disp;
-	unsigned int		 tag_count;
 	const struct got_error	*error;
 	struct got_blob_object	*blob;
 	int			 fd;
@@ -287,7 +280,6 @@ TAILQ_HEAD(addresslist, address);
 
 struct server {
 	TAILQ_ENTRY(server)	 entry;
-	struct addresslist	al;
 
 	char		 name[GOTWEBD_MAXTEXT];
 
@@ -310,11 +302,6 @@ struct server {
 	int		 show_repo_description;
 	int		 show_repo_cloneurl;
 	int		 respect_exportok;
-
-	int		 unix_socket;
-	char		 unix_socket_name[PATH_MAX];
-
-	int		 fcgi_socket;
 };
 TAILQ_HEAD(serverlist, server);
 
@@ -326,9 +313,6 @@ enum client_action {
 struct socket_conf {
 	struct address	 addr;
 
-	char		 name[GOTWEBD_MAXTEXT];
-	char		 srv_name[GOTWEBD_MAXTEXT];
-
 	int		 id;
 	int		 af_type;
 	char		 unix_socket_name[PATH_MAX];
@@ -340,9 +324,6 @@ struct socket {
 	struct socket_conf	 conf;
 
 	int		 fd;
-	int		 pack_fds[GOTWEB_PACK_NUM_TEMPFILES];
-	int		 priv_fd[PRIV_FDS__MAX];
-
 	struct event	 evt;
 	struct event	 ev;
 	struct event	 pause;
@@ -355,7 +336,12 @@ struct passwd;
 struct gotwebd {
 	struct serverlist	servers;
 	struct socketlist	sockets;
+	struct addresslist	addresses;
 
+	int		 pack_fds[GOTWEB_PACK_NUM_TEMPFILES];
+	int		 priv_fd[PRIV_FDS__MAX];
+
+	char		*user;
 	const char	*gotwebd_conffile;
 
 	int		 gotwebd_debug;
@@ -373,9 +359,6 @@ struct gotwebd {
 	int		 server_cnt;
 
 	char		 httpd_chroot[PATH_MAX];
-
-	int		 unix_socket;
-	char		 unix_socket_name[PATH_MAX];
 };
 
 /*
@@ -386,8 +369,6 @@ struct gotweb_url {
 	int		 action;
 	int		 index_page;
 	const char	*commit;
-	const char	*previd;
-	const char	*prevset;
 	const char	*file;
 	const char	*folder;
 	const char	*headref;
@@ -397,8 +378,6 @@ struct gotweb_url {
 struct querystring {
 	uint8_t		 action;
 	char		*commit;
-	char		*previd;
-	char		*prevset;
 	char		*file;
 	char		*folder;
 	char		*headref;
@@ -530,29 +509,7 @@ int config_setserver(struct gotwebd *, struct server *);
 int config_getserver(struct gotwebd *, struct imsg *);
 int config_setsock(struct gotwebd *, struct socket *);
 int config_getsock(struct gotwebd *, struct imsg *);
-int config_setfd(struct gotwebd *, struct socket *);
+int config_setfd(struct gotwebd *);
 int config_getfd(struct gotwebd *, struct imsg *);
 int config_getcfg(struct gotwebd *, struct imsg *);
 int config_init(struct gotwebd *);
-
-/* log.c */
-void	log_init(int, int);
-void	log_procinit(const char *);
-void	log_setverbose(int);
-int	log_getverbose(void);
-void	log_warn(const char *, ...)
-	    __attribute__((__format__ (printf, 1, 2)));
-void	log_warnx(const char *, ...)
-	    __attribute__((__format__ (printf, 1, 2)));
-void	log_info(const char *, ...)
-	    __attribute__((__format__ (printf, 1, 2)));
-void	log_debug(const char *, ...)
-	    __attribute__((__format__ (printf, 1, 2)));
-void	logit(int, const char *, ...)
-	    __attribute__((__format__ (printf, 2, 3)));
-void	vlog(int, const char *, va_list)
-	    __attribute__((__format__ (printf, 2, 0)));
-__dead void fatal(const char *, ...)
-	    __attribute__((__format__ (printf, 1, 2)));
-__dead void fatalx(const char *, ...)
-	    __attribute__((__format__ (printf, 1, 2)));
