@@ -19,6 +19,7 @@
  *     + support indented patches?
  *     + support other kinds of patches?
  */
+#include "got_compat.h"
 
 #include <sys/types.h>
 #include <sys/queue.h>
@@ -48,11 +49,19 @@
 
 #include "got_lib_delta.h"
 #include "got_lib_diff.h"
+#include "got_lib_hash.h"
 #include "got_lib_object.h"
 #include "got_lib_privsep.h"
-#include "got_lib_hash.h"
 
+#ifndef MIN
 #define MIN(a, b) ((a) < (b) ? (a) : (b))
+#endif
+
+struct got_patch_line {
+	char	 mode;
+	char	*line;
+	size_t	 len;
+};
 
 struct got_patch_hunk {
 	STAILQ_ENTRY(got_patch_hunk) entries;
@@ -67,7 +76,7 @@ struct got_patch_hunk {
 	int	new_lines;
 	size_t	len;
 	size_t	cap;
-	char	**lines;
+	struct got_patch_line *lines;
 };
 
 STAILQ_HEAD(got_patch_hunk_head, got_patch_hunk);
@@ -75,8 +84,8 @@ struct got_patch {
 	int	 xbit;
 	char	*old;
 	char	*new;
-	char	 cid[41];
-	char	 blob[41];
+	char	 cid[GOT_HASH_DIGEST_STRING_MAXLEN];
+	char	 blob[GOT_HASH_DIGEST_STRING_MAXLEN];
 	struct got_patch_hunk_head head;
 };
 
@@ -123,7 +132,7 @@ patch_free(struct got_patch *p)
 		STAILQ_REMOVE_HEAD(&p->head, entries);
 
 		for (i = 0; i < h->len; ++i)
-			free(h->lines[i]);
+			free(h->lines[i].line);
 		free(h->lines);
 		free(h);
 	}
@@ -136,7 +145,7 @@ patch_free(struct got_patch *p)
 }
 
 static const struct got_error *
-pushline(struct got_patch_hunk *h, const char *line)
+pushline(struct got_patch_hunk *h, const char *line, size_t len)
 {
 	void 	*t;
 	size_t	 newcap;
@@ -152,10 +161,14 @@ pushline(struct got_patch_hunk *h, const char *line)
 		h->cap = newcap;
 	}
 
-	if ((t = strdup(line)) == NULL)
-		return got_error_from_errno("strdup");
+	if ((t = malloc(len - 1)) == NULL)
+		return got_error_from_errno("malloc");
+	memcpy(t, line + 1, len - 1);	/* skip the line type */
 
-	h->lines[h->len++] = t;
+	h->lines[h->len].mode = *line;
+	h->lines[h->len].line = t;
+	h->lines[h->len].len = len - 2;	/* line type and trailing NUL */
+	h->len++;
 	return NULL;
 }
 
@@ -296,7 +309,7 @@ recv_patch(struct imsgbuf *ibuf, int *done, struct got_patch *p, int strip)
 			}
 
 			if (*t != '\\')
-				err = pushline(h, t);
+				err = pushline(h, t, datalen);
 			else if (lastmode == '-')
 				h->old_nonl = 1;
 			else if (lastmode == '+')
@@ -346,10 +359,10 @@ reverse_patch(struct got_patch *p)
 		h->new_nonl = tmp;
 
 		for (i = 0; i < h->len; ++i) {
-			if (*h->lines[i] == '+')
-				*h->lines[i] = '-';
-			else if (*h->lines[i] == '-')
-				*h->lines[i] = '+';
+			if (h->lines[i].mode == '+')
+				h->lines[i].mode = '-';
+			else if (h->lines[i].mode == '-')
+				h->lines[i].mode = '+';
 		}
 	}
 }
@@ -387,34 +400,36 @@ copy(FILE *tmp, FILE *orig, off_t copypos, off_t pos)
 	return NULL;
 }
 
-static int linecmp(const char *, const char *, int *);
+static int lines_eq(struct got_patch_line *, const char *, size_t, int *);
 
 static const struct got_error *
 locate_hunk(FILE *orig, struct got_patch_hunk *h, off_t *pos, int *lineno)
 {
 	const struct got_error *err = NULL;
+	struct got_patch_line *l = &h->lines[0];
 	char *line = NULL;
-	char mode = *h->lines[0];
+	char mode = l->mode;
 	size_t linesize = 0;
 	ssize_t linelen;
 	off_t match = -1;
 	int mangled = 0, match_lineno = -1;
 
 	for (;;) {
+		(*lineno)++;
 		linelen = getline(&line, &linesize, orig);
 		if (linelen == -1) {
 			if (ferror(orig))
 				err = got_error_from_errno("getline");
-			else if (match == -1)
+			/* An EOF is fine iff the target file is empty. */
+			if (feof(orig) && match == -1 && h->old_lines != 0)
 				err = got_error(GOT_ERR_HUNK_FAILED);
+			match = 0;
+			match_lineno = (*lineno)-1;
 			break;
 		}
-		if (line[linelen - 1] == '\n')
-			line[linelen - 1] = '\0';
-		(*lineno)++;
 
-		if ((mode == ' ' && !linecmp(h->lines[0] + 1, line, &mangled)) ||
-		    (mode == '-' && !linecmp(h->lines[0] + 1, line, &mangled)) ||
+		if ((mode == ' ' && lines_eq(l, line, linelen, &mangled)) ||
+		    (mode == '-' && lines_eq(l, line, linelen, &mangled)) ||
 		    (mode == '+' && *lineno == h->old_from)) {
 			match = ftello(orig);
 			if (match == -1) {
@@ -444,27 +459,34 @@ locate_hunk(FILE *orig, struct got_patch_hunk *h, off_t *pos, int *lineno)
 }
 
 static int
-linecmp(const char *a, const char *b, int *mangled)
+lines_eq(struct got_patch_line *l, const char *b, size_t len, int *mangled)
 {
-	int c;
+	char *a = l->line;
+	size_t i, j;
+
+	if (len > 00 && b[len - 1] == '\n')
+		len--;
 
 	*mangled = 0;
-	c = strcmp(a, b);
-	if (c == 0)
-		return c;
+	if (l->len == len && !memcmp(a, b, len))
+		return 1;
 
 	*mangled = 1;
+
+	i = j = 0;
 	for (;;) {
-		while (*a == '\t' || *a == ' ' || *a == '\f')
-			a++;
-		while (*b == '\t' || *b == ' ' || *b == '\f')
-			b++;
-		if (*a == '\0' || *a != *b)
+		while (i < l->len &&
+		    (a[i] == '\t' || a[i] == ' ' || a[i] == '\f'))
+			i++;
+		while (j < len &&
+		    (b[j] == '\t' || b[j] == ' ' || b[j] == '\f'))
+			j++;
+		if (i == l->len || j == len || a[i] != b[j])
 			break;
-		a++, b++;
+		i++, j++;
 	}
 
-	return *a - *b;
+	return (i == l->len && j == len);
 }
 
 static const struct got_error *
@@ -477,7 +499,7 @@ test_hunk(FILE *orig, struct got_patch_hunk *h)
 	int mangled;
 
 	for (i = 0; i < h->len; ++i) {
-		switch (*h->lines[i]) {
+		switch (h->lines[i].mode) {
 		case '+':
 			continue;
 		case ' ':
@@ -491,9 +513,7 @@ test_hunk(FILE *orig, struct got_patch_hunk *h)
 					    GOT_ERR_HUNK_FAILED);
 				goto done;
 			}
-			if (line[linelen - 1] == '\n')
-				line[linelen - 1] = '\0';
-			if (linecmp(h->lines[i] + 1, line, &mangled)) {
+			if (!lines_eq(&h->lines[i], line, linelen, &mangled)) {
 				err = got_error(GOT_ERR_HUNK_FAILED);
 				goto done;
 			}
@@ -517,13 +537,14 @@ apply_hunk(FILE *orig, FILE *tmp, struct got_patch_hunk *h, int *lineno,
 	size_t linesize = 0, i, new = 0;
 	char *line = NULL;
 	char mode;
+	size_t l;
 	ssize_t linelen;
 
 	if (orig != NULL && fseeko(orig, from, SEEK_SET) == -1)
 		return got_error_from_errno("fseeko");
 
 	for (i = 0; i < h->len; ++i) {
-		switch (mode = *h->lines[i]) {
+		switch (mode = h->lines[i].mode) {
 		case '-':
 		case ' ':
 			(*lineno)++;
@@ -536,18 +557,24 @@ apply_hunk(FILE *orig, FILE *tmp, struct got_patch_hunk *h, int *lineno,
 				if (line[linelen - 1] == '\n')
 					line[linelen - 1] = '\0';
 				t = line;
-			} else
-				t = h->lines[i] + 1;
+				l = linelen - 1;
+			} else {
+				t = h->lines[i].line;
+				l = h->lines[i].len;
+			}
 			if (mode == '-')
 				continue;
-			if (fprintf(tmp, "%s\n", t) < 0) {
+			if (fwrite(t, 1, l, tmp) != l ||
+			    fputc('\n', tmp) == EOF) {
 				err = got_error_from_errno("fprintf");
 				goto done;
 			}
 			break;
 		case '+':
 			new++;
-			if (fprintf(tmp, "%s", h->lines[i] + 1) < 0) {
+			t = h->lines[i].line;
+			l = h->lines[i].len;
+			if (fwrite(t, 1, l, tmp) != l) {
 				err = got_error_from_errno("fprintf");
 				goto done;
 			}
@@ -690,12 +717,14 @@ open_blob(char **path, FILE **fp, const char *blobid,
 	const struct got_error *err = NULL;
 	struct got_blob_object *blob = NULL;
 	struct got_object_id id, *idptr, *matched_id = NULL;
+	enum got_hash_algorithm algo;
 	int fd = -1;
 
 	*fp = NULL;
 	*path = NULL;
+	algo = got_repo_get_object_format(repo);
 
-	if (strlen(blobid) != SHA1_DIGEST_STRING_LENGTH - 1) {
+	if (strlen(blobid) != got_hash_digest_string_length(algo) - 1) {
 		err = got_repo_match_object_id(&matched_id, NULL, blobid,
 		    GOT_OBJ_TYPE_BLOB, NULL /* do not resolve tags */,
 		    repo);
@@ -703,7 +732,7 @@ open_blob(char **path, FILE **fp, const char *blobid,
 			return err;
 		idptr = matched_id;
 	} else {
-		if (!got_parse_object_id(&id, blobid, GOT_HASH_SHA1))
+		if (!got_parse_object_id(&id, blobid, algo))
 			return got_error(GOT_ERR_BAD_OBJ_ID_STR);
 		idptr = &id;
 	}
@@ -768,8 +797,8 @@ prepare_merge(int *do_merge, char **apath, FILE **afile,
 		err = got_object_tree_find_path(&id, NULL, repo, tree, path);
 		if (err)
 			return err;
-		got_sha1_digest_to_str(id->sha1, p->blob, sizeof(p->blob));
-		got_sha1_digest_to_str(commit_id->sha1, p->cid, sizeof(p->cid));
+		got_object_id_hex(id, p->blob, sizeof(p->blob));
+		got_object_id_hex(commit_id, p->cid, sizeof(p->cid));
 		free(id);
 		err = open_blob(apath, afile, p->blob, repo);
 		*do_merge = err == NULL;
@@ -1044,7 +1073,11 @@ got_patch(int fd, struct got_worktree *worktree, struct got_repository *repo,
 		goto done;
 	}
 	imsg_fds[1] = -1;
-	imsg_init(ibuf, imsg_fds[0]);
+	if (imsgbuf_init(ibuf, imsg_fds[0]) == -1) {
+		err = got_error_from_errno("imsgbuf_init");
+		goto done;
+	}
+	imsgbuf_allow_fdpass(ibuf);
 
 	err = send_patch(ibuf, fd);
 	fd = -1;
@@ -1052,7 +1085,7 @@ got_patch(int fd, struct got_worktree *worktree, struct got_repository *repo,
 		goto done;
 
 	err = got_worktree_patch_prepare(&fileindex, &fileindex_path,
-	    worktree);
+	    worktree, repo);
 	if (err)
 		goto done;
 
@@ -1107,9 +1140,8 @@ got_patch(int fd, struct got_worktree *worktree, struct got_repository *repo,
 	}
 
 done:
-	if (fileindex != NULL)
-		complete_err = got_worktree_patch_complete(fileindex,
-		    fileindex_path);
+	complete_err = got_worktree_patch_complete(worktree, fileindex,
+	    fileindex_path);
 	if (complete_err && err == NULL)
 		err = complete_err;
 	free(fileindex_path);
@@ -1120,7 +1152,7 @@ done:
 	if (fd != -1 && close(fd) == -1 && err == NULL)
 		err = got_error_from_errno("close");
 	if (ibuf != NULL)
-		imsg_clear(ibuf);
+		imsgbuf_clear(ibuf);
 	if (imsg_fds[0] != -1 && close(imsg_fds[0]) == -1 && err == NULL)
 		err = got_error_from_errno("close");
 	if (imsg_fds[1] != -1 && close(imsg_fds[1]) == -1 && err == NULL)
