@@ -877,7 +877,7 @@ got_repo_close(struct got_repository *repo)
 	for (i = 0; i < nitems(repo->privsep_children); i++) {
 		if (repo->privsep_children[i].imsg_fd == -1)
 			continue;
-		imsg_clear(repo->privsep_children[i].ibuf);
+		imsgbuf_clear(repo->privsep_children[i].ibuf);
 		free(repo->privsep_children[i].ibuf);
 		err = got_privsep_send_stop(repo->privsep_children[i].imsg_fd);
 		if (err && err->code == GOT_ERR_EOF)
@@ -1474,7 +1474,7 @@ got_repo_list_packidx(struct got_pathlist_head *packidx_paths,
 			break;
 		}
 
-		err = got_pathlist_append(packidx_paths, path_packidx, NULL);
+		err = got_pathlist_insert(NULL, packidx_paths, path_packidx, NULL);
 		if (err)
 			break;
 	}
