@@ -472,7 +472,11 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 		goto done;
 	}
 	imsg_sendfds[1] = -1;
-	imsg_init(&sendibuf, imsg_sendfds[0]);
+	if (imsgbuf_init(&sendibuf, imsg_sendfds[0]) == -1) {
+		err = got_error_from_errno("imsgbuf_init");
+		goto done;
+	}
+	imsgbuf_allow_fdpass(&sendibuf);
 	nsendfd = dup(sendfd);
 	if (nsendfd == -1) {
 		err = got_error_from_errno("dup");
@@ -589,6 +593,7 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 			if (err) {
 				if (err->code != GOT_ERR_NOT_REF)
 					goto done;
+				err = NULL;
 			} else {
 				err = got_ref_resolve(&their_ids[ntheirs],
 				    repo, ref);
