@@ -21,6 +21,7 @@
 #include <sys/uio.h>
 #include <sys/time.h>
 
+#include <err.h>
 #include <stdint.h>
 #include <limits.h>
 #include <signal.h>
@@ -244,7 +245,6 @@ send_gotconfig_remotes(struct imsgbuf *ibuf,
 		return got_error_from_errno("imsg_compose GOTCONFIG_REMOTES");
 
 	err = got_privsep_flush_imsg(ibuf);
-	imsg_clear(ibuf);
 	if (err)
 		return err;
 
@@ -490,7 +490,11 @@ main(int argc, char *argv[])
 #endif
 	signal(SIGINT, catch_sigint);
 
-	imsg_init(&ibuf, GOT_IMSG_FD_CHILD);
+	if (imsgbuf_init(&ibuf, GOT_IMSG_FD_CHILD) == -1) {
+		warn("imsgbuf_init");
+		return 1;
+	}
+	imsgbuf_allow_fdpass(&ibuf);
 
 #ifndef PROFILE
 	/* revoke access to most system calls */
@@ -614,13 +618,13 @@ main(int argc, char *argv[])
 			break;
 	}
 
-	imsg_clear(&ibuf);
 	if (err) {
 		if (!sigint_received && err->code != GOT_ERR_PRIVSEP_PIPE) {
 			fprintf(stderr, "%s: %s\n", getprogname(), err->msg);
 			got_privsep_send_error(&ibuf, err);
 		}
 	}
+	imsgbuf_clear(&ibuf);
 	if (close(GOT_IMSG_FD_CHILD) == -1 && err == NULL)
 		err = got_error_from_errno("close");
 	return err ? 1 : 0;
