@@ -21,9 +21,11 @@
 #include <sys/time.h>
 #include <sys/stat.h>
 
+#include <err.h>
 #include <stdint.h>
 #include <errno.h>
 #include <limits.h>
+#include <poll.h>
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -360,7 +362,8 @@ send_pack(int fd, struct got_pathlist_head *refs,
 		return got_error(GOT_ERR_SEND_EMPTY);
 
 	while (1) {
-		err = got_pkt_readpkt(&n, fd, buf, sizeof(buf), chattygot);
+		err = got_pkt_readpkt(&n, fd, buf, sizeof(buf), chattygot,
+		    INFTIM);
 		if (err)
 			goto done;
 		if (n == 0)
@@ -418,9 +421,14 @@ send_pack(int fd, struct got_pathlist_head *refs,
 		if (err)
 			goto done;
 
-		err = got_pathlist_append(&their_refs, refname, id);
-		if (err)
-			goto done;
+		err = got_pathlist_insert(&pe, &their_refs, refname, id);
+		if (err || pe == NULL) {
+			free(refname);
+			free(id);
+			if (err)
+				goto done;
+		}
+
 
 		if (chattygot)
 			fprintf(stderr, "%s: remote has %s %s\n",
@@ -459,7 +467,7 @@ send_pack(int fd, struct got_pathlist_head *refs,
 			goto done;
 		}
 
-		got_sha1_digest_to_str(their_id->sha1, old_hashstr,
+		got_object_id_hex(their_id, old_hashstr,
 		    sizeof(old_hashstr));
 		got_sha1_digest_to_str(zero_id, new_hashstr,
 		    sizeof(new_hashstr));
@@ -501,14 +509,13 @@ send_pack(int fd, struct got_pathlist_head *refs,
 				}
 				continue;
 			}
-			got_sha1_digest_to_str(their_id->sha1, old_hashstr,
+			got_object_id_hex(their_id, old_hashstr,
 			    sizeof(old_hashstr));
 		} else {
 			got_sha1_digest_to_str(zero_id, old_hashstr,
 			    sizeof(old_hashstr));
 		}
-		got_sha1_digest_to_str(id->sha1, new_hashstr,
-		    sizeof(new_hashstr));
+		got_object_id_hex(id, new_hashstr, sizeof(new_hashstr));
 		err = describe_refchange(&n, &sent_my_capabilites,
 		    my_capabilities, buf, sizeof(buf), refname,
 		    old_hashstr, new_hashstr);
@@ -547,7 +554,7 @@ send_pack(int fd, struct got_pathlist_head *refs,
 			goto done;
 	}
 
-	err = got_pkt_readpkt(&n, fd, buf, sizeof(buf), chattygot);
+	err = got_pkt_readpkt(&n, fd, buf, sizeof(buf), chattygot, INFTIM);
 	if (err)
 		goto done;
 	if (n >= 4 && strncmp(buf, "ERR ", 4) == 0) {
@@ -560,7 +567,8 @@ send_pack(int fd, struct got_pathlist_head *refs,
 	}
 
 	while (nsent > 0) {
-		err = got_pkt_readpkt(&n, fd, buf, sizeof(buf), chattygot);
+		err = got_pkt_readpkt(&n, fd, buf, sizeof(buf), chattygot,
+		    INFTIM);
 		if (err)
 			goto done;
 		if (n < 3) {
@@ -619,7 +627,11 @@ main(int argc, char **argv)
 	TAILQ_INIT(&refs);
 	TAILQ_INIT(&delete_refs);
 
-	imsg_init(&ibuf, GOT_IMSG_FD_CHILD);
+	if (imsgbuf_init(&ibuf, GOT_IMSG_FD_CHILD) == -1) {
+		warn("imsgbuf_init");
+		return 1;
+	}
+	imsgbuf_allow_fdpass(&ibuf);
 #ifndef PROFILE
 	/* revoke access to most system calls */
 	if (pledge("stdio recvfd", NULL) == -1) {
@@ -666,6 +678,7 @@ main(int argc, char **argv)
 	for (i = 0; i < send_req.nrefs; i++) {
 		struct got_object_id *id;
 		char *refname;
+		struct got_pathlist_entry *new;
 
 		if ((err = got_privsep_recv_imsg(&imsg, &ibuf, 0)) != 0) {
 			if (err->code == GOT_ERR_PRIVSEP_PIPE)
@@ -715,13 +728,14 @@ main(int argc, char **argv)
 		}
 		memcpy(id, &href.id, sizeof(*id));
 		if (href.delete)
-			err = got_pathlist_append(&delete_refs, refname, id);
+			err = got_pathlist_insert(&new, &delete_refs, refname, id);
 		else
-			err = got_pathlist_append(&refs, refname, id);
-		if (err) {
+			err = got_pathlist_insert(&new, &refs, refname, id);
+		if (err || new == NULL ) {
 			free(refname);
 			free(id);
-			goto done;
+			if (err)
+				goto done;
 		}
 
 		imsg_free(&imsg);
