@@ -727,7 +727,7 @@ test_clone_multiple_branches() {
 
 	got ref -l -r $testroot/repo-clone > $testroot/stdout
 
-	echo "HEAD: refs/heads/foo" > $testroot/stdout.expected
+	echo "HEAD: refs/heads/bar" > $testroot/stdout.expected
 	echo "refs/heads/bar: $commit_id" >> $testroot/stdout.expected
 	echo "refs/heads/foo: $commit_id" >> $testroot/stdout.expected
 	echo "refs/remotes/origin/bar: $commit_id" \
@@ -748,7 +748,7 @@ remote "origin" {
 	server 127.0.0.1
 	protocol ssh
 	repository "$testroot/repo"
-	branch { "foo" "bar" }
+	branch { "bar" "foo" }
 }
 EOF
 	cmp -s $testroot/repo-clone/got.conf $testroot/got.conf.expected
@@ -768,8 +768,8 @@ EOF
 
 [remote "origin"]
 	url = ssh://127.0.0.1$testroot/repo
-	fetch = refs/heads/foo:refs/remotes/origin/foo
 	fetch = refs/heads/bar:refs/remotes/origin/bar
+	fetch = refs/heads/foo:refs/remotes/origin/foo
 	fetch = refs/tags/*:refs/tags/*
 EOF
 	cmp -s $testroot/repo-clone/config $testroot/config.expected
@@ -867,11 +867,11 @@ test_clone_basic_http() {
 	local testurl=http://127.0.0.1:${GOT_TEST_HTTP_PORT}
 	local commit_id=`git_show_head $testroot/repo`
 
-	timeout 5 ./http-server -p $GOT_TEST_HTTP_PORT $testroot \
+	timeout 10 ./http-server -p $GOT_TEST_HTTP_PORT $testroot \
 	    > $testroot/http-server.log &
 	trap "kill %1" HUP INT QUIT PIPE TERM
 
-	sleep 1 # server starts up
+	sleep 2 # server starts up
 
 	# Test our custom HTTP server with git clone. Should succeed.
 	git clone -q $testurl/repo $testroot/repo-clone-with-git
