@@ -728,7 +728,7 @@ test_log_search()
 
 test_log_mark_keymap()
 {
-	test_init log_mark_keymap 141 10
+	test_init log_mark_keymap 141 14
 
 	local repo="$testroot/repo"
 	local wt="$testroot/wt"
@@ -767,6 +767,7 @@ test_log_mark_keymap()
 	cat <<-EOF >$TOG_TEST_SCRIPT
 	j
 	m		mark commit
+	WAIT_FOR_UI
 	SCREENDUMP
 	EOF
 
@@ -781,6 +782,10 @@ test_log_mark_keymap()
 
 
 
+
+
+
+
 	EOF
 
 	tog log
@@ -797,6 +802,7 @@ test_log_mark_keymap()
 	j
 	m		mark commit
 	m		unmark commit
+	WAIT_FOR_UI
 	SCREENDUMP
 	EOF
 
@@ -811,6 +817,10 @@ test_log_mark_keymap()
 
 
 
+
+
+
+
 	EOF
 
 	tog log
@@ -839,6 +849,10 @@ test_log_mark_keymap()
 
 
 
+
+
+
+
 	EOF
 
 	tog log
@@ -860,7 +874,11 @@ test_log_mark_keymap()
 	EOF
 
 	cat <<-EOF >$testroot/view.expected
-	[1/10] diff $id_head $id_root
+	[1/14] diff $id_head $id_root
+	M  alpha  |  1+  1-
+
+	1 file changed, 1 insertion(+), 1 deletion(-)
+
 	commit - $id_head
 	commit + $id_root
 	blob - $alpha_head
